// STM8A/S		        	        
// AN2857  AN2743        	                
// STM8A/S family power management application note  	        
// Power management lib source file                    
// V1.0.0				        
// THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS  
// WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE   
// TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY       
// DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
// FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE    
// CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.       

 // Includes  **********************
#include "power_management.h"

 // Functions **********************

// WAIT function
void wait (int d){
  while (d--);
}

// Clock configiuration
void config_CLK (void){
  
  // the clock is HSE crystal or external clock      
  #if CLK == HSE
  
  // Switch the clock from HSI to HSE (use automatic switchindg mode)
  // 1) setting the SWEN
  CLK_SWCR |= 0x02;
  // 2) selecting clock source (Select the Fmaster clock as HSE)
  CLK_SWR = 0xB4;
  
  // wait for the switching completion
  while ((CLK_SWCR & 0x08) != 0x08);
  
  // Switch OFF the HSI OSC
  CLK_ICKR &= ~(0x01);
  
  // the clock is HSI                  
  #elif CLK == HSI
  
    #if HSI_DIV == 1
    // fHSI_DIV= fHSI_RC 
    CLK_CKDIVR &= ~(0b00011000);
    
    // fHSI_DIV= fHSI_RC/2 
    #elif HSI_DIV == 2
    CLK_CKDIVR &= ~(0b00011000);
    CLK_CKDIVR |= 0b00001000;
    
    // fHSI_DIV= fHSI_RC/4 
    #elif HSI_DIV == 4
    CLK_CKDIVR &= ~(0b00011000);
    CLK_CKDIVR |= 0b00010000;
    
    // fHSI_DIV= fHSI_RC/8 
    #elif HSI_DIV == 8
    CLK_CKDIVR &= ~(0b00011000);
    CLK_CKDIVR |= 0b00011000;
    
    #else
    #error Please enter a valid division factor for HSI_DIV in the conf.h file. Valid values for HSI_DIV are : 1, 2, 4 or 8
    #endif

  
  // the clock is LSI                  
  #elif CLK == LSI
  // Switch ON the internal low speed oscilator
  CLK_ICKR |= 0x08;
  
  // Switch the clock from HSI to LSI (use automatic switchindg mode)
  // 1) setting the SWEN
  CLK_SWCR |= 0x02;
  // 2) selecting clock source (Select the Fmaster clock as LSI)
  CLK_SWR = 0xD2;
  
  // wait for the switching completion
  while ((CLK_SWCR & 0x08) != 0x08);
  
  // Switch OFF the HSI OSC
  CLK_ICKR &= ~(0x01);
  
  #else
  #error Please select a valid clock for the main clock in the conf.h file. Valid clocks are HSI, HSE or LSI
  #endif
 
  // Select the CPU division factor            
  #if CPU_DIV == 1
  // fCPU= fMASTER 
  CLK_CKDIVR &= ~(0b00000111);
  
  #elif CPU_DIV == 2
  // fCPU= fMASTER/2 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000001;
  
  #elif CPU_DIV == 4
  // fCPU= fMASTER/4 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000010;
  
  #elif CPU_DIV == 8
  // fCPU= fMASTER/8 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000011;
  
  #elif CPU_DIV == 16
  // fCPU= fMASTER/16 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000100;
  
  #elif CPU_DIV == 32
  // fCPU= fMASTER/32 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000101;
  
  #elif CPU_DIV == 64
  // fCPU= fMASTER/64 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000110;
  
  #elif CPU_DIV == 128
  // fCPU= fMASTER/128 
  CLK_CKDIVR &= ~(0b00000111);
  CLK_CKDIVR |= 0b00000111;
  
  #else
  #error Please enter a valid division factor for CPU_DIV in the conf.h file. Valid values for CPU_DIV are: 1, 2, 4, 8, 16, 32, 64 or 128 
  
  #endif
  
  
  // the CPU clock is outputted on CCO pin        
  #ifdef fCPU_on_CCO
  // Fcpu output on MCO pin 
  CLK_CCOR = 0b00001001;
  #endif
  
 // All the peripheral are gated by default 
 // The clock is not provided
 CLK_PCKENR1 = 0x00; //&= ~(0xFF);
 CLK_PCKENR2 = 0x00; //&= ~(0x8C);
}

// Peripheral configuration OFF
void config_periph_OFF(void){
 // Switch the peripherals which are not OFF by default (USART and LINUART)
 USART_CR1 = 0x20;   /*Switch off the usart*/
 LINUART_CR1 = 0x20; /*Switch off the lin uart*/
}

// IO Configuration
void config_GPIO (void){

  // Configure all the IO as Output Push pull at low level exept PA1 and PA2 (OSCIN and OSCOUT)

  // GPIOA
  #if CLK == HSE
  // PA1 and PA2 has to be defined as input floating when HSE is used
  PA_ODR  = GPIO_Value;
  PA_DDR = 0x78;
  PA_CR1 = 0x78;
  #else
  // otherwise it's defined as Push pull at low level 
  PA_ODR  = GPIO_Value;
  PA_DDR = 0x7F;
  PA_CR1 = 0x7F;
  #endif
  
  // Read the CKAWUSEL bit (bit2)  in the OPTION BYTE 4 (OPT4) to determine which clock will be used for the AWU
  switch (OPT4 & 0x04)
  {
    case 0x04 :/*AWU clcok is HSE*/
               // configure PA1 and PA2 as input floating for the oscillator
              PA_ODR  = GPIO_Value;
              PA_DDR &= 0xF8;
              PA_CR1 &= 0xF8; 
              break;
              
    default   :/*AWU clock is LSI*/
               break;
  } 

  // GPIOB
  PB_ODR  = GPIO_Value;
  PB_DDR = 0xFF;
  PB_CR1 = 0xFF;

  // GPIOC
  PC_ODR  = GPIO_Value;
  PC_DDR = 0xFF;
  PC_CR1 = 0xFF;
  
  // GPIOD
  PD_ODR  = GPIO_Value;
  PD_DDR = 0xFF;
  PD_CR1 = 0xFF;
  
  // GPIOE
  PE_ODR  = GPIO_Value;
  PE_DDR = 0xFF;
  PE_CR1 = 0xFF;
  
  // GPIOF
  PF_ODR  = GPIO_Value;
  PF_DDR = 0xFF;
  PF_CR1 = 0xFF;
  
  // GPIOG
  PG_ODR  = GPIO_Value;
  PG_DDR = 0xFF;
  PG_CR1 = 0xFF;
  
  // GPIOH
  PH_ODR  = GPIO_Value;
  PH_DDR = 0xFF;
  PH_CR1 = 0xFF;
  
  // GPIOI
  PI_ODR  = GPIO_Value;
  PI_DDR = 0xFF;
  PI_CR1 = 0xFF;
}

// Configure the flash to be in power down mode during the halt phase of Active HALT mode
void Flash_OFF_act_halt(void){
  FLASH_CR1 |= 0x04;
}

// Configure the flash to be in operating mode during HALT mode
void Flash_ON_halt(void){
  FLASH_CR1 |= 0x08;
}

// Configure the MVR to be OFF during the halt phase of HALT mode
void MVReg_OFF_act_halt(void){
  CLK_ICKR |= 0x20;
}

// Enable the fast clock wake up for fast wake-up from Active HALT mode
void Fast_Clock_WU(void){
  CLK_ICKR |= 0x04;
}

// Configure the AWU unit with the parameters given in the "active_halt_mode.h" file
void config_AWU(unsigned char TBR , unsigned char APR){
 // Enable the clock PCG for AWU
 CLK_PCKENR2 |= 0x04;
 // TBR and APR values are defined in the "active_halt_mode.h" file
 AWU_APR = APR;
 AWU_TBR = TBR;
 // Enable the AWU feature
 AWU_CSR |= 0x10;
}

// Configure external IT on PB5
void config_ext_IT (void){
  // PB5 configured as input floating interrupt
  PB_DDR &= ~(0b00100000);
  PB_CR1 &= ~(0b00100000);
  PB_CR2 |= 0b000100000;
  // Sensitivty is Rising edge only for port B
  EXTI_CR1 = 0x04; 
  
  #ifdef AL_ON
  // The activation level bit is ON 
  CFG_GCR |= 0x02;
  #endif
  
}
